@@ -25,6 +25,7 @@ namespace Aws
             class SubAckPacket;
             class UnsubscribePacket;
             class UnSubAckPacket;
+            class Mqtt5ClientCore;
 
             struct AWS_CRT_CPP_API ReconnectOptions
             {
@@ -146,58 +147,49 @@ namespace Aws
              * Type signature of the callback invoked when connection succeed
              * Mandatory event fields: client, connack_data, settings
              */
-            using OnConnectionSuccessHandler = std::function<void(Mqtt5Client &, const OnConnectionSuccessEventData &)>;
+            using OnConnectionSuccessHandler = std::function<void(const OnConnectionSuccessEventData &)>;
 
             /**
              * Type signature of the callback invoked when connection failed
              */
-            using OnConnectionFailureHandler = std::function<void(Mqtt5Client &, const OnConnectionFailureEventData &)>;
+            using OnConnectionFailureHandler = std::function<void(const OnConnectionFailureEventData &)>;
 
             /**
              * Type signature of the callback invoked when the internal connection is shutdown
              */
-            using OnDisconnectionHandler = std::function<void(Mqtt5Client &, const OnDisconnectionEventData &)>;
+            using OnDisconnectionHandler = std::function<void(const OnDisconnectionEventData &)>;
 
             /**
              * Type signature of the callback invoked when attempting connect to client
              * Mandatory event fields: client
              */
-            using OnAttemptingConnectHandler = std::function<void(Mqtt5Client &, const OnAttemptingConnectEventData &)>;
+            using OnAttemptingConnectHandler = std::function<void(const OnAttemptingConnectEventData &)>;
 
             /**
              * Type signature of the callback invoked when client connection stopped
              * Mandatory event fields: client
              */
-            using OnStoppedHandler = std::function<void(Mqtt5Client &, const OnStoppedEventData &)>;
-
-            /**
-             * Type signature of the callback invoked when a Disconnection Comlete
-             *
-             */
-            using OnDisconnectCompletionHandler = std::function<void(std::shared_ptr<Mqtt5Client>, int)>;
+            using OnStoppedHandler = std::function<void(const OnStoppedEventData &)>;
 
             /**
              * Type signature of the callback invoked when a Publish Complete
              */
-            using OnPublishCompletionHandler =
-                std::function<void(std::shared_ptr<Mqtt5Client>, int, std::shared_ptr<PublishResult>)>;
+            using OnPublishCompletionHandler = std::function<void(int, std::shared_ptr<PublishResult>)>;
 
             /**
              * Type signature of the callback invoked when a Subscribe Complete
              */
-            using OnSubscribeCompletionHandler =
-                std::function<void(std::shared_ptr<Mqtt5Client>, int, std::shared_ptr<SubAckPacket>)>;
+            using OnSubscribeCompletionHandler = std::function<void(int, std::shared_ptr<SubAckPacket>)>;
 
             /**
              * Type signature of the callback invoked when a Unsubscribe Complete
              */
-            using OnUnsubscribeCompletionHandler =
-                std::function<void(std::shared_ptr<Mqtt5Client>, int, std::shared_ptr<UnSubAckPacket>)>;
+            using OnUnsubscribeCompletionHandler = std::function<void(int, std::shared_ptr<UnSubAckPacket>)>;
 
             /**
              * Type signature of the callback invoked when a PacketPublish message received (OnMessageHandler)
              */
-            using OnPublishReceivedHandler = std::function<void(Mqtt5Client &, const PublishReceivedEventData &)>;
+            using OnPublishReceivedHandler = std::function<void(const PublishReceivedEventData &)>;
 
             /**
              * Callback for users to invoke upon completion of, presumably asynchronous, OnWebSocketHandshakeIntercept
@@ -328,73 +320,10 @@ namespace Aws
               private:
                 Mqtt5Client(const Mqtt5ClientOptions &options, Allocator *allocator = ApiAllocator()) noexcept;
 
-                /* Static Callbacks */
-                static void s_publishCompletionCallback(
-                    enum aws_mqtt5_packet_type packet_type,
-                    const void *packet,
-                    int error_code,
-                    void *complete_ctx);
-
-                static void s_subscribeCompletionCallback(
-                    const struct aws_mqtt5_packet_suback_view *puback,
-                    int error_code,
-                    void *complete_ctx);
-
-                static void s_unsubscribeCompletionCallback(
-                    const struct aws_mqtt5_packet_unsuback_view *puback,
-                    int error_code,
-                    void *complete_ctx);
-
-                static void s_lifeCycleEventCallback(const aws_mqtt5_client_lifecycle_event *event);
-
-                static void s_publishReceivedCallback(const aws_mqtt5_packet_publish_view *publish, void *user_data);
-
-                static void s_onWebsocketHandshake(
-                    aws_http_message *rawRequest,
-                    void *user_data,
-                    aws_mqtt5_transform_websocket_handshake_complete_fn *complete_fn,
-                    void *complete_ctx);
-
-                static void s_clientTerminationCompletion(void *complete_ctx);
-
-                /* The handler is set by clientoptions */
-                OnWebSocketHandshakeIntercept websocketInterceptor;
-                /**
-                 * Callback handler trigged when client successfully establishes an MQTT connection
-                 */
-                OnConnectionSuccessHandler onConnectionSuccess;
-
-                /**
-                 * Callback handler trigged when client fails to establish an MQTT connection
-                 */
-                OnConnectionFailureHandler onConnectionFailure;
-
-                /**
-                 * Callback handler trigged when client's current MQTT connection is closed
-                 */
-                OnDisconnectionHandler onDisconnection;
-
-                /**
-                 * Callback handler trigged when client reaches the "Stopped" state
-                 */
-                OnStoppedHandler onStopped;
-
-                /**
-                 * Callback handler trigged when client begins an attempt to connect to the remote endpoint.
-                 */
-                OnAttemptingConnectHandler onAttemptingConnect;
-
-                /**
-                 * Callback handler trigged when an MQTT PUBLISH packet is received by the client
-                 */
-                OnPublishReceivedHandler onPublishReceived;
-                aws_mqtt5_client *m_client;
-                Allocator *m_allocator;
+                /* The client core to handle the user callbacks and c client termination */
+                std::shared_ptr<Mqtt5ClientCore> m_client_core;
 
                 Mqtt5ClientOperationStatistics m_operationStatistics;
-                std::condition_variable m_terminationCondition;
-                std::mutex m_terminationMutex;
-                bool m_terminationPredicate = false;
             };
 
             /**
@@ -404,6 +333,7 @@ namespace Aws
             {
 
                 friend class Mqtt5Client;
+                friend class Mqtt5ClientCore;
 
               public:
                 /**
@@ -418,7 +348,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withHostName(Crt::String hostname);
+                Mqtt5ClientOptions &WithHostName(Crt::String hostname);
 
                 /**
                  * Set port to connect to
@@ -427,7 +357,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withPort(uint16_t port) noexcept;
+                Mqtt5ClientOptions &WithPort(uint16_t port) noexcept;
 
                 /**
                  * Set booststrap for mqtt5 client
@@ -437,7 +367,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withBootstrap(Io::ClientBootstrap *bootStrap) noexcept;
+                Mqtt5ClientOptions &WithBootstrap(Io::ClientBootstrap *bootStrap) noexcept;
 
                 /**
                  * Sets the aws socket options
@@ -446,7 +376,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withSocketOptions(Io::SocketOptions socketOptions) noexcept;
+                Mqtt5ClientOptions &WithSocketOptions(Io::SocketOptions socketOptions) noexcept;
 
                 /**
                  * Sets the tls connection options
@@ -455,7 +385,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withTlsConnectionOptions(const Io::TlsConnectionOptions &tslOptions) noexcept;
+                Mqtt5ClientOptions &WithTlsConnectionOptions(const Io::TlsConnectionOptions &tslOptions) noexcept;
 
                 /**
                  * Sets http proxy options.
@@ -464,7 +394,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withHttpProxyOptions(
+                Mqtt5ClientOptions &WithHttpProxyOptions(
                     const Crt::Http::HttpClientConnectionProxyOptions &proxyOptions) noexcept;
 
                 /**
@@ -474,7 +404,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withConnectOptions(std::shared_ptr<ConnectPacket> packetConnect) noexcept;
+                Mqtt5ClientOptions &WithConnectOptions(std::shared_ptr<ConnectPacket> packetConnect) noexcept;
 
                 /**
                  * Sets session behavior. Overrides how the MQTT5 client should behave with respect to MQTT sessions.
@@ -483,7 +413,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withSessionBehavior(ClientSessionBehaviorType sessionBehavior) noexcept;
+                Mqtt5ClientOptions &WithSessionBehavior(ClientSessionBehaviorType sessionBehavior) noexcept;
 
                 /**
                  * Sets client extended validation and flow control, additional controls for client behavior with
@@ -494,7 +424,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withClientExtendedValidationAndFlowControl(
+                Mqtt5ClientOptions &WithClientExtendedValidationAndFlowControl(
                     ClientExtendedValidationAndFlowControl clientExtendedValidationAndFlowControl) noexcept;
 
                 /**
@@ -507,7 +437,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withOfflineQueueBehavior(
+                Mqtt5ClientOptions &WithOfflineQueueBehavior(
                     ClientOperationQueueBehaviorType offlineQueueBehavior) noexcept;
 
                 /**
@@ -518,7 +448,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withReconnectOptions(ReconnectOptions reconnectOptions) noexcept;
+                Mqtt5ClientOptions &WithReconnectOptions(ReconnectOptions reconnectOptions) noexcept;
 
                 /**
                  * Sets ping timeout (ms). Time interval to wait after sending a PINGREQ for a PINGRESP to arrive.
@@ -528,7 +458,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withPingTimeoutMs(uint32_t pingTimeoutMs) noexcept;
+                Mqtt5ClientOptions &WithPingTimeoutMs(uint32_t pingTimeoutMs) noexcept;
 
                 /**
                  * Sets Connack Timeout (ms). Time interval to wait after sending a CONNECT request for a CONNACK
@@ -538,7 +468,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withConnackTimeoutMs(uint32_t connackTimeoutMs) noexcept;
+                Mqtt5ClientOptions &WithConnackTimeoutMs(uint32_t connackTimeoutMs) noexcept;
 
                 /**
                  * Sets Operation Timeout(Seconds). Time interval to wait for an ack after sending a QoS 1+ PUBLISH,
@@ -548,7 +478,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withAckTimeoutSeconds(uint32_t ackTimeoutSeconds) noexcept;
+                Mqtt5ClientOptions &WithAckTimeoutSeconds(uint32_t ackTimeoutSeconds) noexcept;
 
                 /**
                  * Sets callback for transform HTTP request.
@@ -561,7 +491,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withWebsocketHandshakeTransformCallback(
+                Mqtt5ClientOptions &WithWebsocketHandshakeTransformCallback(
                     OnWebSocketHandshakeIntercept callback) noexcept;
 
                 /**
@@ -571,7 +501,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withClientConnectionSuccessCallback(OnConnectionSuccessHandler callback) noexcept;
+                Mqtt5ClientOptions &WithClientConnectionSuccessCallback(OnConnectionSuccessHandler callback) noexcept;
 
                 /**
                  * Sets callback trigged when client fails to establish an MQTT connection
@@ -580,7 +510,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withClientConnectionFailureCallback(OnConnectionFailureHandler callback) noexcept;
+                Mqtt5ClientOptions &WithClientConnectionFailureCallback(OnConnectionFailureHandler callback) noexcept;
 
                 /**
                  * Sets callback trigged when client's current MQTT connection is closed
@@ -589,7 +519,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withClientDisconnectionCallback(OnDisconnectionHandler callback) noexcept;
+                Mqtt5ClientOptions &WithClientDisconnectionCallback(OnDisconnectionHandler callback) noexcept;
 
                 /**
                  * Sets callback trigged when client reaches the "Stopped" state
@@ -598,7 +528,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withClientStoppedCallback(OnStoppedHandler callback) noexcept;
+                Mqtt5ClientOptions &WithClientStoppedCallback(OnStoppedHandler callback) noexcept;
 
                 /**
                  * Sets callback trigged when client begins an attempt to connect to the remote endpoint.
@@ -607,7 +537,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withClientAttemptingConnectCallback(OnAttemptingConnectHandler callback) noexcept;
+                Mqtt5ClientOptions &WithClientAttemptingConnectCallback(OnAttemptingConnectHandler callback) noexcept;
 
                 /**
                  * Sets callback trigged when a PUBLISH packet is received by the client
@@ -616,7 +546,7 @@ namespace Aws
                  *
                  * @return this option object
                  */
-                Mqtt5ClientOptions &withPublishReceivedCallback(OnPublishReceivedHandler callback) noexcept;
+                Mqtt5ClientOptions &WithPublishReceivedCallback(OnPublishReceivedHandler callback) noexcept;
 
                 /**
                  * Initializes the C aws_mqtt5_client_options from Mqtt5ClientOptions. For internal use
